@@ -9,6 +9,7 @@ class MFAAuth(forms.Form):
     user_id = forms.IntegerField(widget=forms.HiddenInput())
     auth_code = forms.CharField(required=True, widget=forms.TextInput())
     next = forms.CharField(widget=forms.HiddenInput())
+    trusted_device = forms.CharField(widget=forms.CheckboxInput())
 
     def clean(self):
         cleaned_data = super().clean()
@@ -25,16 +26,15 @@ class MFAAuth(forms.Form):
 
         try:
             now = timezone.now()
-            AuthCode.objects.get(user=user)
+            auth = AuthCode.objects.get(user=user)
         except [AuthCode.DoesNotExist, AuthCode.MultipleObjectsReturned]:
             self.add_error("auth_code", "The code for your account was not found. Please request a new one.")
         else:
-            auth = AuthCode.objects.get(user=user)
             if auth.expires <= now:
                 self.add_error("auth_code", "Your code has expired. Please request a new one.")
                 auth.delete()
             elif not check_password(auth_code.upper(), auth.code):
-                self.add_error("auth_code", "An invalid code was entered. Please try again or request a new one.")
+                self.add_error("auth_code", "We were unable to authenticate the code provided.")
 
     def authenticate(self):
         if self.is_valid():
