@@ -16,10 +16,14 @@ class ValidateMFAMiddleware:
         if getattr(view_func, 'login_exempt', False):
             return None
 
+        if "_simplemfa_trusted_device" in request.COOKIES:
+            return None
+
         if 'simplemfa' in request.resolver_match.namespaces:
             return None
 
-        if request.path in [reverse("simplemfa:mfa-login"), reverse("simplemfa:mfa-request")]:
+        if request.path in [reverse("simplemfa:mfa-login"), reverse("simplemfa:mfa-request"),
+                            reverse("login"), reverse("logout")]:
             return None
 
         if request.user.is_authenticated and mfa_required and not mfa_authenticated:
