@@ -7,9 +7,10 @@ from simplemfa.models import AuthCode
 from django.core.exceptions import PermissionDenied
 from django.http import HttpResponseBadRequest
 from django.http import JsonResponse
-from simplemfa.helpers import template_fallback, get_user_mfa_mode
 from django.contrib import messages
-from simplemfa.helpers import send_mfa_code
+from django.utils import timezone
+from simplemfa.helpers import send_mfa_code, get_user_mfa_mode, get_user_phone, parse_phone, set_cookie, get_cookie_expiration, \
+    sanitize_email, sanitize_phone, template_fallback, build_mfa_request_url
 
 
 class MFALoginView(LoginRequiredMixin, TemplateView):
@@ -22,63 +23,50 @@ class MFALoginView(LoginRequiredMixin, TemplateView):
         initial = {"user_id": request.user.id, "next": next_url}
         context['form'] = self.form_class(initial=initial)
         context['next'] = next_url
+        context['mfa_code_sent'] = request.session.get("_simplemfa_code_sent", False)
+        context['default_mode'] = get_user_mfa_mode(request)
+        context['trusted_device_days'] = get_cookie_expiration()
+        context['request_url'] = build_mfa_request_url(request)
 
-        # if this session key is not set, we send a code - this prevents new codes being generated on page refresh
-        if not request.session.get("_simplemfa_code_sent", False):
+        email = request.user.email
+        context['sanitized_email'] = sanitize_email(email)
 
-            try:
-                # delete old codes (if any) for this user
-                AuthCode.delete_all_codes_for_user(request.user.id)
-
-                # the selected mode for the user to get the code
-                mode = get_user_mfa_mode(request)
-
-                # create the new MFA auth object
-                code = AuthCode.create_code_for_user(request.user.id, sent_via=mode)
-
-                # if code is good, process it
-                if code is not None:
-                    request.session['_simplemfa_code_sent'] = True
-
-                    # eventually replace this with send_mfa_code(request, code, mode) where mode is how it is to be sent
-                    send_mfa_code(request, code, mode=mode)
-
-                    # for testing in development
-                    if settings.DEBUG:
-                        print("MFA CODE: " + code)
-
-                else:
-                    raise Exception
-
-            except Exception:
-                AuthCode.delete_all_codes_for_user(request.user.id)
-                request.session['_simplemfa_code_sent'] = False
-                messages.add_message(request, messages.ERROR,
-                                     "Something went wrong. A code was not created. Try again.")
+        phone = sanitize_phone(get_user_phone(request))
+        context['sanitized_phone'] = phone
 
         return render(request, self.get_template_names(), context)
 
     def post(self, request, *args, **kwargs):
         form_data = self.form_class(request.POST)
-        next_url = request.GET.get("next", None)
-
-        # the form authenticates the code provided, returns True if it checks out
         if form_data.authenticate():
-
-            # replace the original next_url with data from the form (if any)
             next_url = form_data.cleaned_data.get("next", request.GET.get("next", None))
-
-            # this authenticates the user, letting the middleware know they have passed verification
             request.session['_simplemfa_authenticated'] = True
-
             if next_url is not None:
-                return redirect(next_url, request)
+                response = redirect(next_url, request)
             else:
-                # where do we go after being authenticated if not set in next_url?
-                redirect_view = settings.LOGIN_REDIRECT_URL if hasattr(settings, 'LOGIN_REDIRECT_URL') else "home"
-                return redirect(reverse(redirect_view), request)
+                redirect_view = settings.LOGIN_REDIRECT_URL if hasattr(settings, 'LOGIN_REDIRECT_URL') else "index"
+                response = redirect(reverse(redirect_view), request)
+            if form_data.cleaned_data.get("trusted_device").upper() == "TRUE":
+                set_cookie(response, "_simplemfa_trusted_device", timezone.now())
+            return response
         else:
-            context = {"form": form_data, "form_errors": form_data.errors, "next": next_url}
+            request.session['_simplemfa_authenticated'] = False
+            context = self.get_context_data()
+            context['mfa_code_sent'] = request.session.get("_simplemfa_code_sent", False)
+            context['userid'] = request.user.id
+            context['next'] = form_data.cleaned_data.get("next", request.GET.get("next", None))
+            context['form'] = form_data
+            context['default_mode'] = get_user_mfa_mode(request)
+            context['trusted_device_days'] = get_cookie_expiration()
+            context['request_url'] = build_mfa_request_url(request)
+
+            email = request.user.email
+            context['sanitized_email'] = sanitize_email(email)
+
+            phone = sanitize_phone(get_user_phone(request))
+            context['sanitized_phone'] = phone
+
+            messages.add_message(request, messages.ERROR, "We were unable to authenticate the code provided")
             return render(request, self.get_template_names(), context)
 
     def get_template_names(self):
@@ -95,39 +83,51 @@ OUTPUT: JSON response (result)
 class MFARequestView(LoginRequiredMixin, View):
 
     def get(self, request, *args, **kwargs):
-        # variables
-        mode = request.GET.get("sent_via", get_user_mfa_mode(request))
 
-        # start doing some work
+        reset = request.GET.get("reset", None)
         response = {}
-        try:
-            # delete old codes (if any) for this user
+        if reset is not None and reset.upper() == "TRUE":
             AuthCode.delete_all_codes_for_user(request.user.id)
+            request.session['_simplemfa_code_sent'] = False
+        else:
+            # variables
+            mode = request.GET.get("sent_via", get_user_mfa_mode(request))
 
-            # create the new MFA auth object
-            code = AuthCode.create_code_for_user(request.user.id, sent_via=mode)
+            try:
+                # delete old codes (if any) for this user
+                AuthCode.delete_all_codes_for_user(request.user.id)
 
-            # send email or text to the user, depending on selected mode
-            send_mfa_code(request, code, mode=mode)
+                # create the new MFA auth object
+                code = AuthCode.create_code_for_user(request.user.id, sent_via=mode)
 
-            # for testing in development
-            if settings.DEBUG:
-                print("MFA CODE: " + code)
+                # send email or text to the user, depending on selected mode
+                send_mfa_code(request, code, mode=mode)
 
-            # the result is good, code is created and sent
-            response['code_created'] = True
+                # for testing in development
+                if settings.DEBUG:
+                    print("MFA CODE: " + code)
 
-        except:
-            # something went wrong, let them know no new code was issued
-            AuthCode.delete_all_codes_for_user(request.user.id)
-            response['code_created'] = False
-            messages.add_message(request, messages.ERROR, "Something went wrong. A code was not created. Try again.")
+                if code is not None:
+                    messages.add_message(request, messages.SUCCESS, "A new code has been sent.")
+                    request.session['_simplemfa_code_sent'] = True
+                    # the result is good, code is created and sent
+                    response['code_created'] = True
+            except:
+                # something went wrong, let them know no new code was issued
+                AuthCode.delete_all_codes_for_user(request.user.id)
+                response['code_created'] = False
+                request.session['_simplemfa_code_sent'] = False
+                messages.add_message(request, messages.ERROR,
+                                     "Something went wrong. A code was not created. Try again.")
 
         if request.is_ajax():
             return JsonResponse(response)
         else:
-            messages.add_message(request, messages.SUCCESS, "A new code has been sent.")
-            return redirect(reverse("simplemfa:mfa-login"), request)
+            next_url = request.GET.get("next", None)
+            url = reverse("mfa:mfa-login")
+            if next_url is not None:
+                url += f"?next={next_url}"
+            return redirect(url, request)
 
     def post(self, request, *args, **kwargs):
         # request must be AJAX
@@ -160,14 +160,17 @@ class MFARequestView(LoginRequiredMixin, View):
             if settings.DEBUG:
                 print("MFA CODE: " + code)
 
-            # the result is good
-            response['code_created'] = True
-            response['message'] = "A new code was created and has been sent."
+            if code is not None:
+                request.session['_simplemfa_code_sent'] = True
+                # the result is good
+                response['code_created'] = True
+                response['message'] = "A new code was created and has been sent."
 
         except:
             # something went wrong, let them know no new code was issued or sent
             AuthCode.delete_all_codes_for_user(request.user.id)
             response['code_created'] = False
+            request.session['_simplemfa_code_sent'] = False
             response['message'] = "Something went wrong and we were unable to generate a new code. Try again."
 
         return JsonResponse(response)
